/**
 * FreeBridge is a Java-Based application to play bridge on your computer.
 * Copyright (C) Axelle and Ludovic Apvrille
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this 
 * program; if not, write to the 
 *	 Free Software Foundation, Inc., 59 Temple Place, 
 *	 Suite 330, Boston, MA 02111-1307 USA
 *
 */
package freebridge.console;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.MissingResourceException;
import org.apache.log4j.Category;
import org.apache.log4j.PropertyConfigurator;
import freebridge.core.*;
import freebridge.international.*;

/**
 * @author Axelle Apvrille
 * @version 0.1
 */
public class ConsoleBridge {
    static Category cat = Category.getInstance(ConsoleBridge.class.getName());

    public static final String BANNER = "Freebridge 0.1 -- Console Game";
    private Match currentMatch = null;
    private Player p[];
    private International intl;
    private BufferedReader reader;

    public ConsoleBridge(){
	this(new International());
    }

    public ConsoleBridge(International intl){ 
	cat.debug("> ConsoleBridge(): intl="+intl);
	this.intl = intl; 
	System.out.println(BANNER);
	reader = new BufferedReader(new InputStreamReader(System.in));
	p = new Player[4];
	cat.debug("< ConsoleBridge()");
    }

    protected void finalize(){
	cat.debug("> finalize()");
	try {
	    reader.close();
	}
	catch(IOException exp){
	    cat.error("IOException caught: "+exp);
	}
	cat.debug("< finalize()");
    }

    /**
     * asks whether players are human or AI.
     */
    public void setPlayers(){
	cat.debug("> setPlayer()");
	int i=0;
	while (i<p.length){
	    try {
		System.out.print(intl.getString("ENTER_PLAYER_NAME"));
		String name = reader.readLine();

		System.out.print(intl.getString("ASK_AI_HUMAN"));
		String choice = reader.readLine();

		if (choice.equals("1")) {
		    p[i] = new ConsolePlayer(name);
		}
		else {
		    // TODO: AI Player
		    System.out.println(intl.getString("NOT_IMPLEMENTED"));
		    // REPLACE once AI PLayer exists
		    p[i] = new ConsolePlayer(name);
		}
		i++;
	    }
	    catch(MissingResourceException exp1){
		cat.fatal("Exception caught: "+exp1);
		System.exit(-1);
	    }
	    catch(Exception exp){
		// Not fatal. Can be an input error.
		cat.error("Exception caught: "+exp);
	    // TODO: display error.
	    }
	}
	cat.debug("< setPlayer()");
    }

    /**
     * creates a new match with the players that have been set.
     * @throws NullPointerException if the players have not been set...
     */
    public void initMatch(){
	cat.debug("> initMatch()");
	currentMatch = new Match();
	currentMatch.setNorth(p[0]);
	currentMatch.setEast(p[1]);
	currentMatch.setSouth(p[2]);
	currentMatch.setWest(p[3]);
	cat.debug("< initMatch()");
    }

    public void play() throws MissingResourceException {
	cat.debug("> play()");
	
	do {
	    System.out.println(intl.getString("STARTING_NEW_GAME"));

	    Game g = new Game(currentMatch);

	    // let's deal the cards
	    cat.info("Dealing the cards...");
	    g.doDeal();

	    // let's do the bid stage
	    cat.info("Starting the bid stage...");
	    Bid toplay = g.doBidStage(intl);

	    // show what we're playing
	    if (toplay != null && (toplay.getValue()!=Bid.PASS ||
				   toplay.getSuit()!=Bid.IGNORED ||
				   toplay.getInsult()!=Bid.IGNORED)){
		cat.info("We're going to play: "+toplay+" ");
		System.out.print(intl.getString("BID_TO_PLAY")+" ");
		System.out.println(toplay.show(intl));
	    }

	    // next game
	    cat.info("End of game");
	    System.out.println(intl.getString("ENDING_GAME"));
	    currentMatch.nextGame();
	} while(currentMatch.getDealer().getOrientation() != Orientation.NORTH);

	cat.debug("< play()");
    }

    public static void main(String args[]){
	PropertyConfigurator.configure(args[0]);
	cat.info("Starting console game");

	ConsoleBridge theGame = new ConsoleBridge();
	theGame.setPlayers();
	theGame.initMatch();
	try {
	    theGame.play();
	}
	catch(MissingResourceException exp){
	    cat.fatal("Exception caught: "+exp);
	}
	cat.info("End.");
    }

}
