/**
 * FreeBridge is a Java-Based application to play bridge on your computer.
 * Copyright (C) Axelle and Ludovic Apvrille
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this 
 * program; if not, write to the 
 *	 Free Software Foundation, Inc., 59 Temple Place, 
 *	 Suite 330, Boston, MA 02111-1307 USA
 *
 */
package freebridge.console;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.MissingResourceException;
import org.apache.log4j.Category;
import freebridge.core.*;
import freebridge.international.International;

/**
 * This is a human player for which all UI uses the console.
 * @author Axelle Apvrille
 * @version 0.1
 */
public class ConsolePlayer extends Player {
    static Category cat = Category.getInstance(ConsolePlayer.class.getName());
    static private final String EXIT = "EXIT"; // escase answer to get out the game
    private BufferedReader reader;

    public ConsolePlayer(String n) { 
	super(n); 
	reader = new BufferedReader(new InputStreamReader(System.in));
    }

   protected void finalize(){
	cat.debug("> finalize()");
	try {
	    reader.close();
	}
	catch(IOException exp){
	    cat.error("IOException caught: "+exp);
	}
	cat.debug("< finalize()");
    }

    /**
     * Asks for the bid of the player.
     * TO DO: improve so that the player knows what are the impossible bids
     * also, accept various inputs and put default values.
     * @param intl language to print messages
     * @return bid of the player
     * @throws BadTypeException if the player enters bad input for his bid
     * @throws IOException if we cannot read/write on the console
     * @throws MissingResourceException if we cannot find the correct translation for messages
     * to write on the console.
     */
    public Bid doBid(International intl) 
	throws BadTypeException,IOException,MissingResourceException {
	cat.debug("> doBid()");
	String insult,suit,value;
	Bid thebid = new Bid();

	// get the suit
	System.out.print(intl.getString("ASK_BID_SUIT"));
	suit = reader.readLine();
	cat.debug("suit read from console="+suit);

	if (suit.equalsIgnoreCase(EXIT))
	    System.exit(0);
	    
	if (suit.equalsIgnoreCase(intl.getString("CLUBS_SHORT")))
	    thebid.setSuit(Card.CLUBS);
	
	if (suit.equalsIgnoreCase(intl.getString("DIAMOND_SHORT")))
	    thebid.setSuit(Card.DIAMOND);
	
	if (suit.equalsIgnoreCase(intl.getString("HEART_SHORT")))
	    thebid.setSuit(Card.HEART);
	
	if (suit.equalsIgnoreCase(intl.getString("SPADES_SHORT")))
	    thebid.setSuit(Card.SPADES);
	
	if (suit.equalsIgnoreCase(intl.getString("NOTRUMP_SHORT")))
	    thebid.setSuit(Bid.NOTRUMP);

	cat.debug("Bid="+thebid);

	// get the value
	System.out.print(intl.getString("ASK_BID_VALUE"));
	value = reader.readLine();
	cat.debug("value read from console="+value);
	if (value.equalsIgnoreCase(EXIT))
	    System.exit(0);

	if (value.equalsIgnoreCase(intl.getString("PASS")))
	    thebid.setValue(Bid.PASS);
	else {
	    try {
		thebid.setValue(Integer.valueOf(value).intValue());
	    }
	    catch(NumberFormatException exp){
		cat.warn("NumberFormatException caught: "+exp+" throwing BadTypeException");
		throw new BadTypeException("Value is not a number");
	    }
	}

	// get the insult
	System.out.print(intl.getString("ASK_BID_INSULT"));
	insult = reader.readLine();
	cat.debug("insult read from console="+insult);
	if (insult.equalsIgnoreCase(EXIT))
	    System.exit(0);
	
	if (insult.equalsIgnoreCase(intl.getString("DOUBLE")))
	    thebid.setInsult(Bid.DOUBLE);
	
	if (insult.equalsIgnoreCase(intl.getString("REDOUBLE")))
	    thebid.setInsult(Bid.REDOUBLE);

	cat.debug("Bid="+thebid);

	cat.debug("< doBid(): thebid="+thebid);
	return thebid;
    }

    public Card doPlay(){
	// TODO
	return null;
    }

    public void showHand(International intl) throws MissingResourceException {
	cat.debug("> showHand()");
	Hand myhand = getHand();
	System.out.println("--- "+this.getName()+" "+this.getOrientation().show(intl));
	System.out.println(myhand.showHand(intl));
	cat.debug("< showHand()");
    }
}
