package freebridge.core;

import freebridge.international.International;
import org.apache.log4j.Category;
import java.util.MissingResourceException;

/**
 * Card of a standard bridge game. No jokers.
 * @author Axelle Apvrille
 * @version 0.1
 */
public class Card {
    static Category cat = Category.getInstance(Card.class.getName());

    /**
     * The various bridge colours. They are set in the bridge order.
     */
    public static final int CLUBS = 1;
    public static final int DIAMOND = 2;
    public static final int HEART = 3;
    public static final int SPADES = 4;

    /**
     * The various card numbers. The 1 is the ace. But it's the highest card in HonourValue
     */
    public static final int ACE = 1;
    public static final int KING = 13;
    public static final int QUEEN = 12;
    public static final int JACK = 11;

    /**
     * Provided for help within this class
     */
    private static final int MIN_VALUE = 1;
    private static final int MAX_VALUE = 13;
    
    private int value = 1;
    private int colour = CLUBS;

    // TODO: add image file for each card

    /**
     * Creates a card.
     * @param value the card number, from 1 to 13. May use the ACE, KING, QUEEN and JACK constants too.
     * @param colour the suit of the card.
     * @throws BadTypeException if you set wrong input values.
     */ 
    public Card(int value, int colour) 
	throws BadTypeException {
	cat.debug("> Card(): value = " + value + " colour = "+ colour);

	if ((value < MIN_VALUE) || (value > MAX_VALUE))
	    throw new BadTypeException("Unknown card value");
	if ((colour < CLUBS) || (colour > SPADES))
	    throw new BadTypeException("Unknown card colour");
	this.value = value;
	this.colour = colour;
	cat.debug("< Card()");
    }

    public int getValue() { return value; }

    /**
     * @return the suit of the card.
     */
    public int getColour() { return colour; }

    public static boolean isMinor(int colour) { 
	if ((colour == CLUBS) || (colour == DIAMOND))
	    return true;
	return false;
    }

    public static boolean isMajor(int colour){
	return (!isMinor(colour));
    }

    /**
     * @return Honour points associated with this card. Ace is the highest with 4. Down to Jack 1, and the rest 0.
     */
    public int getHonourValue() {
	cat.debug("getHonourValue(): card is = "+value);
	if (value == Card.JACK) return 1;
	if (value == Card.QUEEN) return 2;
	if (value == Card.KING) return 3;
	if (value == Card.ACE) return 4;
	return 0;
    }

    /*
     * @return number of the card (in letters).
     */
    public static String showCardValue(International intl, int value) throws MissingResourceException{
	cat.debug("> showCardValue(): value = "+ value);
	String s = "";
	if ((value > Card.ACE ) && (value < 11)) {
	    Integer i = new Integer(value);
	    s = i.toString();
	}

	if (value == Card.ACE)
	    s = intl.getString("ACE_SHORT");
	if (value == Card.JACK)
	    s = intl.getString("JACK_SHORT");
	if (value == Card.QUEEN)
	    s = intl.getString("QUEEN_SHORT");
	if (value == Card.KING)
	    s = intl.getString("KING_SHORT");

	if (! s.equals(""))
	    cat.debug("< showCardValue(): value = "+value+" string= "+s);
	else
	    cat.error("< showCardValue(): value = "+value+" unknown tag");
	return s;
    }

    public String showCardValue(International intl) throws MissingResourceException {
	return showCardValue(intl,this.value);
    }

    /**
     * @return colour of the card, in a short string. (abreviated). Depends on your country.
     */
    public static String showCardColour(International intl, int colour) throws MissingResourceException{
	cat.debug("> showCardColour(): colour = "+colour);
	String s = "";

	switch(colour){
	case CLUBS:
	    s = intl.getString("CLUBS_SHORT");
	    break;
	case DIAMOND:
	    s = intl.getString("DIAMOND_SHORT");
	    break;
	case HEART:
	    s = intl.getString("HEART_SHORT");
	    break;
	case SPADES:
	    s = intl.getString("SPADES_SHORT");
	    break;
	default:
	    cat.error("Unknown tag");
	    break;
	}

	cat.debug("< showCardColour(): colour = "+ colour+ " string= "+s);
	return s;
    }

    public String showCardColour(International intl) throws MissingResourceException {
	return showCardColour(intl,this.colour);
    }

    /**
     * For debugging
     */
    public String toString(){
	return "Value = " + value + " Colour=" + colour;
    }

    public boolean equals(Object obj){
	cat.debug("Card.equals()");
	if (obj instanceof Card) {
	    Card c = (Card) obj;
	    if ((c.getColour()==colour) && (c.getValue()==value))
		return true;
	} 
	return false;
    }
}
