package freebridge.core;

import org.apache.log4j.Category;
import java.util.Vector;
import java.util.Iterator;
import java.util.MissingResourceException;
import freebridge.international.International;

/**
 * A hand of cards. 13 cards in bridge. Remembers cards you have in hand, and also those that have been played.
 * @author Axelle Apvrille
 * @version 0.1
 */
public class Hand {
    static Category cat = Category.getInstance(Hand.class.getName());

    private Vector cards;
    private boolean playedCards[];
    private static final int NUMBER = 13;

    public Hand(){
	cat.debug(">Hand()");
	cards = new Vector(Hand.NUMBER);
	playedCards = new boolean[Hand.NUMBER];
	cat.debug("<Hand()");
    }

    public void resetHand(){
	cat.debug("> resetHand()");
	cards.clear();
	for (int i=0;i<Hand.NUMBER;i++)
	    playedCards[i] = false;
	cat.debug("< resetHand()");
    }

    /**
     * add a card to your hand. The card is marked 'not played'.
     * @param c card to add
     * @throws ArrayIndexOutOfBoundsException if you already have 13 cards in your hand
     * @throws ImpossibleActionException if you're trying to add a card that already exists (cheater !)
     */
    public void addCard(Card c) throws ArrayIndexOutOfBoundsException, ImpossibleActionException {
	cat.debug("> Hand.addCard(): card is "+c.toString());
	if (cards.size()>= Hand.NUMBER) throw new ArrayIndexOutOfBoundsException();
	if (cards.indexOf(c) != -1) throw new ImpossibleActionException(ImpossibleActionException.CARD_ALREADY_EXISTS);
	cards.add(c);
	playedCards[cards.indexOf((Object)c)]=false;
	cat.debug("< Hand.addCard()");
    }

    /**
     * @return all cards not played of a given colour. If not cards, then an empty Vector is returned.
     */
    public Vector selectColour(int colour){
	cat.debug("> selectColour(): colour="+colour);
	Vector v = new Vector(Hand.NUMBER);
	for (int i=0; i< cards.size() ; i++) {
	    if (playedCards[i] == false){
		Card c = (Card) cards.get(i);
		if (c.getColour() == colour) {
		    cat.debug("Selected: "+c.toString());
		    v.add(c);
		}
	    }
	}
	cat.debug("< selectColour()");
	return v;
    }

    /**
     * Marks a card as played. Cannot be played again.
     * @param c card to play
     * @throws ImpossibleActionException if you're trying to play a card that you don't have, or a card
     * that has already been played.
     */
    public void playCard(Card c) throws ImpossibleActionException {
	cat.debug("> playCard(): card="+c.toString());
	int index = cards.indexOf(c);
	if (index == -1) throw new ImpossibleActionException(ImpossibleActionException.CARD_DOES_NOT_EXIST);
	if (playedCards[index]==true)
	    throw new ImpossibleActionException(ImpossibleActionException.CARD_ALREADY_PLAYED);
	playedCards[index]=true;
	cat.debug("< playCard()");
    }

    /**
     * Displays the cards you haven't played yet in your hand
     * Cards are displayed colour by colour.
     * @param intl language to use
     * @throws MissingResourceException if we cannot find the correct translation for the cards.
     */
    public String showHand(International intl) throws MissingResourceException {
	cat.debug("> showHand()");
	String display = "";
	
	for (int i=Card.CLUBS; i<=Card.SPADES; i++){
	    Vector colour = selectColour(i);
	    Iterator j = colour.iterator();
	    display += Card.showCardColour(intl,i) + " ";
	    while (j.hasNext()){
		Card c = (Card) j.next();
		display += c.showCardValue(intl);
		display += " ";
	    }
	    display += "\n";
	}

	cat.debug("< showHand(): "+display);
	return display;
    }

    public String toString(){
	cat.debug("> toString()");
	Iterator i = cards.iterator();
	int j=0;
	String s = "";
	while (i.hasNext()) {
	    Card c = (Card)i.next();
	    String thecard = c.toString();
	    s = s+ "c["+j+"]= "+thecard;
	    s = s+ "p["+j+"]= "+playedCards[j];
	    cat.debug("Card["+j+"]= "+thecard+" played="+playedCards[j]);
	    j++;
	}
	cat.debug("< toString()"); 
	return s; 
    }
}
