package freebridge.test;

import org.apache.log4j.Category;
import junit.framework.*;
import freebridge.core.*;

public class BidStageTest extends TestCase {
    static Category cat = Category.getInstance(BidStageTest.class.getName());
    
    public static Test suite(){
	cat.debug("====================== BID STAGE TESTS ====================");
	return new TestSuite(BidStageTest.class);
    }

    public void testGetSet(){
	try {
	    cat.debug("We builds a bid stage object with dealer at north");
	    Orientation dealer = new Orientation(Orientation.NORTH);
	    BidStage stage = new BidStage(dealer);
	    Assert.assertEquals(dealer,stage.getDealer());
	    Assert.assertTrue(stage.getBids().isEmpty());
	    Assert.assertEquals(Orientation.NORTH,stage.getNextOrientation().getOrientation());
	}
	catch(Exception exp){
	    cat.error("Unexpected excecption caught: "+exp);
	    fail("Unexpected excecption caught: "+exp);
	}
    }

    public void testAddBid(){
	try {
	    Orientation dealer = new Orientation(Orientation.EAST);
	    BidStage stage = new BidStage(dealer);
	    Bid b1 = new Bid();
	    b1.setValue(Bid.PASS);
	    cat.debug("EAST Passes");
	    stage.addBid(b1);
	    cat.info("Stage is : "+stage.toString());
	    Assert.assertEquals(Orientation.SOUTH,stage.getNextOrientation().getOrientation());
	    Assert.assertEquals(1,stage.getBids().size());
	    Assert.assertEquals(b1,stage.getLastBid());

	    Bid b2 = new Bid();
	    b2.setInsult(Bid.DOUBLE); // this is valid
	    cat.debug("SOUTH doubles");
	    stage.addBid(b2);
	    Assert.assertEquals(2,stage.getBids().size());

	    Bid b3 = new Bid();
	    b3.setValue(1);
	    b3.setSuit(Bid.NOTRUMP);
	    stage.addBid(b3);

	    Bid b4 = new Bid();
	    b4.setValue(2);
	    b4.setSuit(Card.CLUBS);
	    stage.addBid(b4);

	    Bid b5 = new Bid();
	    b5.setValue(2);
	    b5.setSuit(Card.HEART);
	    stage.addBid(b5);

	    Bid b6 = new Bid();
	    b6.setValue(2);
	    b6.setInsult(Bid.DOUBLE);
	    stage.addBid(b6);

	    //TODO : tests that fail
	}
	catch(Exception exp){
	    cat.error("Unexpected excecption caught: "+exp);
	    fail("Unexpected excecption caught: "+exp);
	}
    }

    public void testIsFinished(){
	cat.debug("Test the isFinished()");
	try {
	    Orientation dealer = new Orientation(Orientation.EAST);
	    BidStage stage = new BidStage(dealer);
	    Assert.assertFalse(stage.isFinished());

	    Bid b1 = new Bid();
	    b1.setValue(Bid.PASS);
	    cat.debug("EAST Passes");
	    stage.addBid(b1);
	    Assert.assertFalse(stage.isFinished());

	    cat.debug("South says 1NT");
	    Bid b3 = new Bid();
	    b3.setValue(1);
	    b3.setSuit(Bid.NOTRUMP);
	    stage.addBid(b3);
	    Assert.assertFalse(stage.isFinished());

	    cat.debug("West says PASS");
	    stage.addBid(b1);

	    cat.info("North says PASS");
	    stage.addBid(b1);
	    Assert.assertFalse(stage.isFinished());

	    cat.info("East says PASS");
	    stage.addBid(b1);
	    Assert.assertFalse(stage.isFinished());

	    cat.info("South says PASS");
	    stage.addBid(b1);
	    Assert.assertTrue(stage.isFinished());
	}
	catch(Exception exp){
	    cat.error("Unexpected excecption caught: "+exp);
	    fail("Unexpected excecption caught: "+exp);
	}
    }

    public void testGetBidToPlay(){
	cat.info("Testing the getBidToPlay() method");
	try {
	    Orientation dealer = new Orientation(Orientation.EAST);
	    BidStage stage = new BidStage(dealer);
	    cat.info("Testing without any bid");
	    Assert.assertEquals(null,stage.getBidToPlay());

	    Bid b1 = new Bid();
	    b1.setValue(Bid.PASS);
	    cat.info("EAST Passes");
	    stage.addBid(b1);
	    Assert.assertEquals(null,stage.getBidToPlay());
	    
	    cat.debug("South says 1NT");
	    Bid b2 = new Bid();
	    b2.setValue(1);
	    b2.setSuit(Bid.NOTRUMP);
	    stage.addBid(b2);
	    Assert.assertEquals(1,stage.getBidToPlay().getValue());
	    Assert.assertEquals(Bid.NOTRUMP,stage.getBidToPlay().getSuit());
	    Assert.assertEquals(Bid.IGNORED,stage.getBidToPlay().getInsult());

	    cat.debug("West doubles");
	    Bid b3 = new Bid();
	    b3.setInsult(Bid.DOUBLE);
	    stage.addBid(b3);
	    Assert.assertEquals(1,stage.getBidToPlay().getValue());
	    Assert.assertEquals(Bid.NOTRUMP,stage.getBidToPlay().getSuit());
	    Assert.assertEquals(Bid.DOUBLE,stage.getBidToPlay().getInsult());



	}
	catch(Exception exp){
	    cat.error("Unexpected excecption caught: "+exp);
	    fail("Unexpected excecption caught: "+exp);
	}
    }

    public void testWashedInsult(){
	cat.info("Testing a case where the insult is washed away by another bid.");
	try {
	    Orientation dealer = new Orientation(Orientation.SOUTH);
	    BidStage stage = new BidStage(dealer);
	    Bid b1 = new Bid();
	    Bid b2 = new Bid();
	    Bid b3 = new Bid();
	    Bid b4 = new Bid();
	    b1.setValue(1);
	    b1.setSuit(Card.CLUBS);
	    b2.setInsult(Bid.DOUBLE);
	    b3.setValue(2);
	    b3.setSuit(Card.HEART);
	    b4.setValue(Bid.PASS);
	    stage.addBid(b1);
	    stage.addBid(b2);
	    stage.addBid(b3);
	    stage.addBid(b4);
	    stage.addBid(b4);
	    stage.addBid(b4);
	    stage.addBid(b4);
	    cat.info("We should be playing 2 hearts.");
	    Assert.assertEquals(2,stage.getBidToPlay().getValue());
	    Assert.assertEquals(Card.HEART,stage.getBidToPlay().getSuit());
	    Assert.assertEquals(Bid.IGNORED,stage.getBidToPlay().getInsult());
	}
	catch(Exception exp){
	    cat.error("Unexpected excecption caught: "+exp);
	    fail("Unexpected excecption caught: "+exp);
	}
    }

    public void testGetDead(){
	cat.info("Test the getDead() method");
	try {
	    Orientation dealer = new Orientation(Orientation.WEST);
	    BidStage stage = new BidStage(dealer);
	    cat.info("Testing without any bid");
	    Assert.assertEquals(null,stage.getDead());

	    cat.info("West says 1 heart");
	    Bid b1 = new Bid();
	    b1.setValue(1);
	    b1.setSuit(Card.HEART);
	    stage.addBid(b1);
	    Assert.assertEquals(null,stage.getDead());

	    cat.info("North passes");
	    Bid b2 = new Bid();
	    b2.setValue(Bid.PASS);
	    stage.addBid(b2);
	    Assert.assertEquals(null,stage.getDead());

	    cat.info("East says 2 heart");
	    Bid b3 = new Bid();
	    b3.setValue(2);
	    b3.setSuit(Card.HEART);
	    stage.addBid(b3);
	    Assert.assertEquals(null,stage.getDead());

	    cat.info("The rest passes");
	    stage.addBid(b2);
	    stage.addBid(b2);
	    stage.addBid(b2);
	    stage.addBid(b2);
	    Assert.assertEquals(Orientation.EAST,stage.getDead().getOrientation());
	}
	catch(Exception exp){
	    cat.error("Unexpected excecption caught: "+exp);
	    fail("Unexpected excecption caught: "+exp);
	}
    }
}
